<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Carbon;

class FunctionsController extends Controller{
    public function __construct(){}

    public static function uuidv5($namespace, $name) {
        if(!self::is_valid($namespace)) return false;

        // Get hexadecimal components of namespace
        $nhex = str_replace(array('-','{','}'), '', $namespace);

        // Binary Value
        $nstr = '';

        // Convert Namespace UUID to bits
        for($i = 0; $i < strlen($nhex); $i+=2) {
            $nstr .= chr(hexdec($nhex[$i].$nhex[$i+1]));
        }

        // Calculate hash value
        $hash = sha1($nstr . $name);

        return sprintf('%08s-%04s-%04x-%04x-%12s',

            // 32 bits for "time_low"
            substr($hash, 0, 8),

            // 16 bits for "time_mid"
            substr($hash, 8, 4),

            // 16 bits for "time_hi_and_version",
            // four most significant bits holds version number 5
            (hexdec(substr($hash, 12, 4)) & 0x0fff) | 0x5000,

            // 16 bits, 8 bits for "clk_seq_hi_res",
            // 8 bits for "clk_seq_low",
            // two most significant bits holds zero and one for variant DCE1.1
            (hexdec(substr($hash, 16, 4)) & 0x3fff) | 0x8000,

            // 48 bits for "node"
            substr($hash, 20, 12)
        );
    }

    public static function is_valid($uuid) {
        return preg_match('/^\{?[0-9a-f]{8}\-?[0-9a-f]{4}\-?[0-9a-f]{4}\-?'.
                        '[0-9a-f]{4}\-?[0-9a-f]{12}\}?$/i', $uuid) === 1;
    }

    public function generateID($seed, $timestamp){
        $string = str_replace(" ", "", $seed) . $timestamp;
        $array = [];
        for($i = 0; $i < strlen($string); $i++){
            $array[] = $string[$i];
        }

        shuffle($array);
        $chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        
        $ID = '';
        foreach($array as $char){
            $ascii = ord($char);

            $s = $ascii / 100;
            if($s >= 1) continue;

            $s = $s * strlen($chars);
            $s = intval($s);

            $l = $chars[$s];
            if(strlen($ID) < 20) $ID .= $l;
        }

        if(strlen($ID) < 20){
            for($i = strlen($ID); $i < 20; $i++){
                $s = rand(0, strlen($chars) - 1);
                $l = $chars[$s];
                $ID .= $l;
            }
        }
        
        return $ID;
    }

    public function getType($type){
        switch($type){
            case "insert":
                return "CREATE";
            break;
            case "update":
                return strtoupper($type);
            break;
            case "select":
                return "READ";
            break;
            case "delete":
                return strtoupper($type);
            break;
            default:
                return "UNKNOWN";
            break;
        }
    }

    public function registerLog($querys, $user, $date, $idac, $line){
        foreach($querys as $query){
            $script = $query['query'];
            $params = json_encode($query['bindings']);

            DB::table('S002V01TLOMY')->insert([
                'LOMY_NULI' => $line,
                'LOMY_IDAC' => $idac,
                'LOMY_QUER' => $script,
                'LOMY_PARA' => $params,
                'LOMY_FECH' => $date,
                'LOMY_IDUS' => $user,
            ]);
        }
    }

    public function registerActivity($line, $module, $function, $screen, $action, $description, $user, $date, $submodule = null){
        $id = DB::table('S002V01TACCI')->insertGetId([
            'ACCI_NULI' => $line,
            'ACCI_IDMO' => $module,
            'ACCI_IDSM' => $submodule,
            'ACCI_IDFU' => $function,
            'ACCI_IDPA' => $screen,
            'ACCI_TIAC' => $action,
            'ACCI_DESC' => $description,
            'ACCI_IDUS' => $user,
            'ACCI_FEAC' => $date
        ]);

        DB::table('S002V01TUSUA')->where('USUA_IDUS', '=', $user)->update([
            'USUA_ULAC' => $id
        ]);

        return $id;
    }

    public function joinName($name, $fApe, $sApe = null){
        $fullName = "$name $fApe";
        if(!is_null($sApe)){
            $fullName = "$fullName $sApe";
        }

        return $fullName;
    }

    public function checkFileSize($ext, $size){
        $availableFiles = [
            250=> ["zip", "tar", "rar", "7z", "mp3", "mpeg", "wav", "ogg", "opus", "jpg", "png", "gif", "bmp", "tiff", "svg",
            "txt", "webm", "mp4", "3gp", "mov", "avi", "wmv", "flv", "dwg", "dxf", "3ds", "ai", "psd"],
            50 => ["doc", "docx", "vsd", "vsdx", "pdf"],
            75 => ["xls", "xlsx"],
            100=> ["ppt", "pptx"],
        ];

        $maxFileSizeMB = 0;
        foreach($availableFiles as $k=>$v){
            if(in_array($ext, $v)) $maxFileSizeMB = $k;
        }

        if($maxFileSizeMB == 0) return false;

        $maxFileSize = $maxFileSizeMB * 1048576;
        return $size < $maxFileSize;
    }
}